package mappingrobot.bluetoothappnote;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.UUID;

public class MainActivity extends AppCompatActivity {
    // UUID for Serial Port Profile used by Bluetooth Chip
    private final static UUID SPP_UUID = UUID.fromString(
            "00001101-0000-1000-8000-00805F9B34FB"
    );

    // Used to indicate start and end of packet
    private final static byte START_PACKET = 0x2;
    private final static byte END_PACKET = 0x3;

    private List<BluetoothDevice> devices = new ArrayList<>();


    private OutputStream output;
    private InputStream input;
    private BluetoothSocket socket;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        // NOTE:
        // Ensure that you have the following in your activity layout (activity_main.xml)
        //      a Button with the id "refresh"
        //      a ListView with the id "devices"


        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Before beginning:
        //      Ensure bluetooth is enabled on your phone
        //      Ensure that the bluetooth device is paired with your phone

        scan();

        Button refresh = (Button) findViewById(R.id.refresh);
        refresh.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                scan();
            }
        });

        final AppCompatActivity me = this;
        ListView v = (ListView) findViewById(R.id.devices);
        v.setOnItemClickListener(new ListView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                BluetoothDevice d = devices.get(position);
                Log.i("Bluetooth", "ID: " + d.getAddress());
                Log.i("Bluetooth", "Name: " + d.getName());
                try {
                    socket = d.createInsecureRfcommSocketToServiceRecord(SPP_UUID);
                    socket.connect();

                    output = socket.getOutputStream();
                    input = socket.getInputStream();

                    byte[] toWrite = "TEST MESSAGE".getBytes();
                    output.write(START_PACKET);
                    output.write(toWrite);
                    output.write(END_PACKET);

                    byte[] replyBuf = new byte[256];

                    // NOTES:
                    //  1) You'll have to do checking for START_PACKET and END_PACKET here as well
                    //      Packets sometimes take multiple read()s to get here - concatenate based
                    //      on START_PACKET and END_PACKET
                    //  2) read() is a blocking call. This should be done in a separate thread
                    //      to allow the ui thread to keep running.
                    //  3) Depending on the performance of your bluetooth chip / android phone, bytes may
                    //      be missing, so check for that here as well.
                    int bytesReceived = input.read(replyBuf);
                    String reply = new String(Arrays.copyOfRange(replyBuf, 0, bytesReceived));

                    Toast.makeText(me, "reply: " + reply, Toast.LENGTH_LONG).show();

                    output.close();
                    input.close();
                    socket.close();

                } catch(IOException e) {
                    e.printStackTrace();
                    Toast.makeText(me, "Couldn't connect", Toast.LENGTH_LONG).show();
                }

            }
        });
    }

    public void scan() {
        try {
            BluetoothAdapter adapter = BluetoothAdapter.getDefaultAdapter();
            Set<BluetoothDevice> pairedDevices = adapter.getBondedDevices();

            List<String> s = new ArrayList<>();
            devices.clear();
            for (BluetoothDevice bt : pairedDevices) {
                s.add(bt.getName());
                devices.add(bt);
            }

            ListView list = (ListView) findViewById(R.id.devices);

            list.setAdapter(new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, s));
        }
        catch(Exception e) {
            Toast.makeText(this, e.getMessage(), Toast.LENGTH_LONG).show();
        }
    }
}
